require File.expand_path(File.dirname(__FILE__) + '/../../../lib/TNR360/components/dependencies.rb')

class LdsBlock

  @browser
  @idBlock=nil
  @idScreen=nil
  @title=nil
  @visible=false
  @editable=false
  @exists=false
  @collapsible=false
  @collapsed=false
  @hasMultipleAreas=false
  @areas
  @type=nil
  @paginated_grid=false
  @current_element

  @idLine=nil
  @idCel=nil
  @celValue=nil


  #constructor
  def initialize(browser, idBlock, idScreen=nil)
    @browser =browser
    @idBlock=idBlock
    @idScreen=idScreen
    update
    @exists
  end


  #refresh object status from browser
  def refresh
    #Look for element in browser
    @current_element= findElement @idBlock
    #update other data
    update
    @exists
  end

  #private method to update/save the status of the object
  def update
    @exists=false

    #Look for element in browser
    @current_element||= findElement @idBlock

    # fill hasMultipleAreas
    @hasMultipleAreas=(@current_element.div(:index => 2).fieldsets.length>1 ? true : false)

    if (@hasMultipleAreas)
      #extract areas if available
      extractAreas
    else
      #fill @title
      extractTitle
      #fill visible
      @visible=@current_element.visible?
      #fill type
      extractType
      #fill isCollapsible
      @collapsible=@current_element.div(:index => 0).div(:index => 0).div.exists?
      #fill isCollapsed
      @collapsed=@current_element.class_name.include? "collapsed"
    end
    #everything exists
    @exists=true
  end


  #internal methods

  def extractAreas
    if (@hasMultipleAreas)
      @areas=Array.new
      #iterate through list of areas
      @current_element.fieldsets.each do |fieldset|
        @areas << LdsArea.new(@browser, fieldset.id.to_s, self)
      end
    end
  end


  def extractType
    if (@current_element.class_name.include? "form")
      @type="Form"
    else
      @type="Grid"
    end
  end

  def extractTitle
    if @current_element.div(:index => 0).spans.length>0
      @title=@current_element.div(:index => 0).span(:index => 0).text
    else
      @title=""
    end
  end

  #method locate current element
  def findElement idElement
    @current_element=nil

    #wait for screen to become visible (find screen forcefully)
    proc=Proc.new { isScreenDisplayed }
    force_find_element(proc).to_s

    #wait for screen to become visible (find screen forcefully)
    proc=Proc.new { elementExists?(idElement) }
    @exists=force_find_element(proc)
    puts 'Block found = '+@exists.to_s

    if (@browser.divs(:id => idElement).length>1)
      @browser.divs(:id => idElement).each do |field|
        if field.visible?
          @current_element=field
          break
        end
      end
    else
      @current_element=@browser.div(:id => idElement).when_present
    end


    if @current_element.visible?
      puts 'block is visible'
    else
      puts 'block is NOT visible'
    end
    @current_element


  end


  def elementExists?(idElement)
    #this method assumes the screen/tab is already loaded
    oneIsVisible=false
    #if element not yet available in screen, wait for some time
    if (!@browser.div(:id => idElement).exists?)
      @browser.div(:id => idElement).when_present(10)
      oneIsVisible=true
    else
      if (!@browser.div(:id => idElement).visible?)
        @browser.divs(:id => idElement).each do |field|
          if field.visible?
            oneIsVisible=true
            break
          end
        end
        if !oneIsVisible
          @browser.div(:id => idElement).when_present(10)
          oneIsVisible=true
        end
      else
        oneIsVisible=true
      end
    end
    oneIsVisible
  end

  def isScreenDisplayed
    if (@idScreen!=nil)
      @scr=LdsScreen.new(@browser, @idScreen)
      @scr.isVisible?
    else
      false
    end
  end


  #print object
  def to_s
    if (@hasMultipleAreas && @areas!=nil)
      @areas.each do |area|
        area.to_s
      end
    else
      "\n***** Block *****"+
          "\nIdBlock : "+ no_null(@idBlock)+
          "\nIdScreen : "+ no_null(@idScreen) +
          "\nTitle : "+ no_null(@title)+
          "\nCollapsible :"+ bool_no_null(@collapsible.to_s)+
          "\nCollapsed :"+ bool_no_null(@collapsed.to_s)+
          "\nVisible : "+ bool_no_null(@visible.to_s)+
          "\nExists : "+ bool_no_null(@exists.to_s)+
          "\nHasMultipleAreas :"+ bool_no_null(@hasMultipleAreas.to_s)+
          "\nType :"+ no_null(@type.to_s)+
          "\n**********"
    end
  end


  ##LdsScreen Actions
  def reverse_collapse
    if (@collapsible)
      @current_element.div(:index => 0).div(:index => 0).div.when_present.click
    end
    update
  end

  def collapse
    if (@collapsible && !@collapsed)
      @current_element.div(:index => 0).div(:index => 0).div.when_present.click
    end
    update
  end

  def expand
    if (@collapsible && @collapsed)
      @current_element.div(:index => 0).div(:index => 0).div.when_present.click
    end
    update
  end

  ####Getters

  def getElement
    @current_element
  end

  def getIdBlock
    @idBlock
  end

  def getIdScreen
    @idScreen
  end

  def getTitle
    @title
  end

  def getAreas
    @areas
  end

  def getType
    @type
  end

  def hasMultipleAreas?
    @hasMultipleAreas
  end

  def isCollapsible?
    @collapsible
  end

  def isCollapsed?
    @collapsed
  end

  def isVisible?
    @visible
  end

  def isExist?
    @exists
  end

  #private method list
  private :update, :extractTitle, :extractType
end




